@@ -1,7 +1,7 @@
 /*
  * PCRE - Perl Compatible Regular Expressions
  * <http://www.pcre.org/>
- * Copyright (C) 2003-2005 Sam Steingold
+ * Copyright (C) 2003-2007 Sam Steingold
  * GPL2
  */
 
@@ -30,7 +30,7 @@
 DEFMODULE(pcre,"PCRE")
 
 DEFUN(PCRE::PCRE-VERSION,)
-{ VALUES3(asciz_to_string(pcre_version(),GLO(misc_encoding)),
+{ VALUES3(safe_to_string(pcre_version()),
           fixnum(PCRE_MAJOR),fixnum(PCRE_MINOR)); }
 #if defined(HAVE_PCRE_CONFIG)
 DEFCHECKER(pcre_config_option, prefix=PCRE_CONFIG, UTF8 NEWLINE LINK-SIZE \
@@ -99,7 +99,7 @@ DEFUN(PCRE:PCRE-COMPILE,string &key :STUDY :IGNORE-CASE :MULTILINE :DOTALL \
     });
   if (compiled_pattern == NULL) { /* error */
     pushSTACK(NIL);               /* no PLACE */
-    pushSTACK(asciz_to_string(error_message,GLO(misc_encoding)));
+    pushSTACK(safe_to_string(error_message));
     pushSTACK(fixnum(error_offset));
     pushSTACK(*string); pushSTACK(TheSubr(subr_self)->name);
     check_value(error,GETTEXT("~S(~S) at ~S: ~S"));
@@ -142,12 +142,12 @@ static void check_pattern (object pat, pcre** compiled_pattern,
 }
 
 /* two objects should be on STACK for the error message */
-DEFCHECKER(pcre_error_code,prefix=PCRE_ERROR, NOMATCH NULL BADOPTION    \
+DEFCHECKER(error_pcre_code,prefix=PCRE_ERROR, NOMATCH NULL BADOPTION    \
            BADMAGIC UNKNOWN_NODE NOMEMORY NOSUBSTRING MATCHLIMIT CALLOUT \
            DFA_UITEM DFA_UCOND DFA_UMLIMIT DFA_WSSIZE DFA_RECURSE       \
            BADUTF8 BADUTF8_OFFSET PARTIAL BADPARTIAL INTERNAL BADCOUNT)
-nonreturning_function(static, pcre_error, (int status)) {
-  pushSTACK(pcre_error_code_reverse(status));
+nonreturning_function(static, error_pcre, (int status)) {
+  pushSTACK(error_pcre_code_reverse(status));
   pushSTACK(sfixnum(status)); pushSTACK(TheSubr(subr_self)->name);
   fehler(error,"~S/~S=~S: ~S ~S");
 }
@@ -157,7 +157,7 @@ nonreturning_function(static, pcre_error, (int status)) {
     begin_system_call();                                \
     status = pcre_fullinfo(c_pat,study,opt,val);        \
     end_system_call();                                  \
-    if (status < bad) pcre_error(status);               \
+    if (status < bad) error_pcre(status);               \
   } while(0)
 DEFCHECKER(fullinfo_arg,prefix=PCRE_INFO,OPTIONS SIZE CAPTURECOUNT      \
            BACKREFMAX FIRSTBYTE FIRSTTABLE LASTLITERAL                  \
@@ -169,16 +169,16 @@ DEFCHECKER(fullinfo_arg,prefix=PCRE_INFO,OPTIONS SIZE CAPTURECOUNT      \
        tion.  External  callers  can  cause PCRE to use its internal tables by
        passing a NULL table pointer. */
 static object fullinfo_options (pcre *c_pat, pcre_extra *study) {
-  unsigned long int options, nn = 0;
+  unsigned long int options;
   int status;
   PCRE_INFO(PCRE_INFO_OPTIONS,&options,0);
   return pcre_options_to_list(options);
 }
-static object fullinfo_size (pcre *c_pat, pcre_extra *study) {
-  unsigned long int length;
+static object fullinfo_size (pcre *c_pat, pcre_extra *study, int opt) {
+  size_t length;
   int status;
-  PCRE_INFO(PCRE_INFO_SIZE,&length,0);
-  return UL_to_I(length);
+  PCRE_INFO(opt,&length,0);
+  return size_to_I(length);
 }
 static object fullinfo_int (pcre *c_pat, pcre_extra *study, int opt) {
   int ret, status;
@@ -203,7 +203,7 @@ static object fullinfo_firsttable (pcre *c_pat, pcre_extra *study) {
   handle_fault_range(PROT_READ_WRITE,(aint)data,(aint)data + sizeof(table));
   begin_system_call();
   status = pcre_fullinfo(c_pat,study,PCRE_INFO_FIRSTTABLE,&table);
-  if (status < 0) { end_system_call(); pcre_error(status); }
+  if (status < 0) { end_system_call(); error_pcre(status); }
   memcpy(data,table,sizeof(table));
   end_system_call();
   return ret;
@@ -220,16 +220,16 @@ static object fullinfo_nametable (pcre *c_pat, pcre_extra *study) {
   char *table;
   begin_system_call();
   status = pcre_fullinfo(c_pat,study,PCRE_INFO_NAMECOUNT,&count);
-  if (status < 0) { end_system_call(); pcre_error(status); }
+  if (status < 0) { end_system_call(); error_pcre(status); }
   status = pcre_fullinfo(c_pat,study,PCRE_INFO_NAMEENTRYSIZE,&size);
-  if (status < 0) { end_system_call(); pcre_error(status); }
+  if (status < 0) { end_system_call(); error_pcre(status); }
   status = pcre_fullinfo(c_pat,study,PCRE_INFO_NAMETABLE,&table);
   end_system_call();
-  if (status < 0) pcre_error(status);
+  if (status < 0) error_pcre(status);
   for (pos = 0; pos < count; pos++, table+=size) {
     pushSTACK(allocate_cons());
     Car(STACK_0) = asciz_to_string(table+2,GLO(misc_encoding));
-    Cdr(STACK_0) = fixnum((table[0]<<1) + table[1]);
+    Cdr(STACK_0) = fixnum((table[0]<<8) + table[1]);
   }
   return listof(count);
 }
@@ -242,11 +242,10 @@ DEFUN(PCRE:PATTERN-INFO,pattern &optional request)
   if (missingp(STACK_0)) {
     int count = 0;
     pushSTACK(`:OPTIONS`); pushSTACK(fullinfo_options(c_pat,study)); count+=2;
-    pushSTACK(`:SIZE`); pushSTACK(fullinfo_size(c_pat,study)); count+=2;
+    pushSTACK(`:SIZE`);
+    pushSTACK(fullinfo_size(c_pat,study,PCRE_INFO_SIZE)); count+=2;
     pushSTACK(`:CAPTURECOUNT`);
     pushSTACK(fullinfo_int(c_pat,study,PCRE_INFO_CAPTURECOUNT)); count+=2;
-    pushSTACK(`:BACKREFMAX`);
-    pushSTACK(fullinfo_int(c_pat,study,PCRE_INFO_BACKREFMAX)); count+=2;
 #  if defined(PCRE_INFO_FIRSTBYTE)
     pushSTACK(`:FIRSTBYTE`);pushSTACK(fullinfo_firstbyte(c_pat,study));count+=2;
 #  endif
@@ -268,7 +267,7 @@ DEFUN(PCRE:PATTERN-INFO,pattern &optional request)
 #  endif
 #  if defined(PCRE_INFO_STUDYSIZE)
     pushSTACK(`:STUDYSIZE`);
-    pushSTACK(fullinfo_int(c_pat,study,PCRE_INFO_STUDYSIZE)); count+=2;
+    pushSTACK(fullinfo_size(c_pat,study,PCRE_INFO_STUDYSIZE)); count+=2;
 #  endif
 #  if defined(PCRE_INFO_NAMECOUNT) && defined(PCRE_INFO_NAMEENTRYSIZE) && defined(PCRE_INFO_NAMETABLE)
     pushSTACK(`:NAMETABLE`);
@@ -279,7 +278,8 @@ DEFUN(PCRE:PATTERN-INFO,pattern &optional request)
     int arg = fullinfo_arg(STACK_0);
     switch (arg) {
       case PCRE_INFO_OPTIONS: VALUES1(fullinfo_options(c_pat,study)); break;
-      case PCRE_INFO_SIZE: VALUES1(fullinfo_size(c_pat,study)); break;
+      case PCRE_INFO_SIZE:
+        VALUES1(fullinfo_size(c_pat,study,PCRE_INFO_SIZE)); break;
       case PCRE_INFO_CAPTURECOUNT: case PCRE_INFO_BACKREFMAX:
 #    if defined(PCRE_INFO_NAMEENTRYSIZE)
       case PCRE_INFO_NAMEENTRYSIZE:
@@ -355,7 +355,7 @@ DEFUN(PCRE:PCRE-EXEC,pattern subject &key :WORK-SPACE :DFA :BOOLEAN :OFFSET \
   begin_system_call();
   ret = pcre_fullinfo(c_pat,study,PCRE_INFO_CAPTURECOUNT,&capture_count);
   end_system_call();
-  if (ret < 0) pcre_error(ret);
+  if (ret < 0) error_pcre(ret);
   /* when DFA is enabled, the number of matches is not easily estimated */
   ovector_size = 3 * (capture_count + (dfa_p ? workspace_size : 1));
  pcre_exec_retry:
@@ -396,7 +396,7 @@ DEFUN(PCRE:PCRE-EXEC,pattern subject &key :WORK-SPACE :DFA :BOOLEAN :OFFSET \
         }
       VALUES1(popSTACK());
     }
-  } else pcre_error(ret);
+  } else error_pcre(ret);
   skipSTACK(2);                 /* drop pattern & subject */
 }
 
