@@ -1,6 +1,6 @@
 ;;;; Common Lisp Object System for CLISP: Generic Functions
 ;;;; Bruno Haible 21.8.1993 - 2004
-;;;; Sam Steingold 1998 - 2004
+;;;; Sam Steingold 1998 - 2004, 2007
 ;;;; German comments translated into English: Stefan Kain 2002-04-08
 
 (in-package "CLOS")
@@ -137,17 +137,17 @@
                  all-keys))))
 
 ;; Preliminary.
-(defun ensure-generic-function-using-class (gf funname &rest args
-                                            &key generic-function-class
-                                                 lambda-list
-                                                 argument-precedence-order
-                                                 method-class
-                                                 method-combination
-                                                 documentation
-                                                 declarations
-                                                 declare
-                                                 environment
-                                            &allow-other-keys)
+(predefun ensure-generic-function-using-class (gf funname &rest args
+                                               &key generic-function-class
+                                                    lambda-list
+                                                    argument-precedence-order
+                                                    method-class
+                                                    method-combination
+                                                    documentation
+                                                    declarations
+                                                    declare
+                                                    environment
+                                               &allow-other-keys)
   (declare (ignore generic-function-class lambda-list argument-precedence-order
                    method-class method-combination documentation declarations
                    declare environment))
@@ -215,7 +215,7 @@
     `(LET ()
        (COMPILER::EVAL-WHEN-COMPILE
          (COMPILER::C-DEFUN ',funname ,signature nil 'DEFMETHOD))
-       (WHEN (GET ,(if (atom funname) `',funname `(SYS::GET-SETF-SYMBOL ',(second funname))) 'SYS::TRACED-DEFINITION)
+       (WHEN (GET ',(SYS::GET-FUNNAME-SYMBOL funname) 'SYS::TRACED-DEFINITION)
          (SYS::UNTRACE1 ',funname))
        (DO-DEFMETHOD ',funname (FUNCTION ,fast-function-factory-lambda)
                      (LIST ,@method-initargs-forms)))))
@@ -527,14 +527,17 @@
       (analyze-defgeneric 'defgeneric whole-form funname lambda-list options)
     (let* ((generic-function-class-var (gensym))
            (generic-function-class-keywords-var (gensym))
+           (funname-symbol (sys::get-funname-symbol funname))
            (method-combination-form `(,method-combination-lambda ,generic-function-class-var)))
       `(LET ()
          (DECLARE (SYS::IN-DEFUN ,funname))
          (COMPILER::EVAL-WHEN-COMPILE
            (COMPILER::C-DEFUN ',funname ',signature nil 'DEFGENERIC))
-         (WHEN (GET ,(if (atom funname) `',funname `(SYS::GET-SETF-SYMBOL ',(second funname))) 'SYS::TRACED-DEFINITION)
+         (WHEN (GET ',funname-symbol 'SYS::TRACED-DEFINITION)
            (SYS::UNTRACE1 ',funname))
          ;; NB: no (SYSTEM::REMOVE-OLD-DEFINITIONS ',funname)
+         (sys::check-redefinition ',funname 'defgeneric
+                                  (sys::fbound-string ',funname-symbol))
          (LET* ((,generic-function-class-var ,generic-function-class-form)
                 ,@(if user-defined-args
                     `((,generic-function-class-keywords-var
