@@ -2,7 +2,7 @@
 ;;;; 1.8.1989, 2.9.1989, 8.10.1989
 
 (in-package "EXT")
-(export '(doseq dohash without-package-lock memoized))
+(export '(expand-form doseq dohash without-package-lock memoized))
 
 (export '(#-(or UNIX WIN32) custom::*default-time-zone*
           custom::*system-package-list*)
@@ -11,6 +11,10 @@
 
 (in-package "SYSTEM")
 
+;;; code walker
+(defun expand-form (form &aux *fenv* *venv*)
+  (%expand-form form))
+
 ;;; functions for symbols (Chapter 10)
 
 (defun copy-symbol (symbol &optional flag) ;; Common LISP, p. 169
@@ -44,7 +48,7 @@
 (defmacro do-symbols ((var &optional (packageform '*package*) (resultform nil))
                       &body body)
   (multiple-value-bind (body-rest declarations) (system::parse-body body)
-    (let ((packvar (gensym)))
+    (let ((packvar (gensym "PACKAGE-")))
       `(BLOCK NIL
          (LET ((,packvar ,packageform))
            (LET ((,var NIL))
@@ -60,7 +64,7 @@
                                     (resultform nil))
                                &body body)
   (multiple-value-bind (body-rest declarations) (system::parse-body body)
-    (let ((packvar (gensym)))
+    (let ((packvar (gensym "PACKAGE-")))
       `(BLOCK NIL
          (LET ((,packvar ,packageform))
            (LET ((,var NIL))
@@ -101,8 +105,8 @@
            (TEXT "~S: flag must be one of the symbols ~S, ~S, ~S, not ~S")
            'with-package-iterator ':internal ':external ':inherited
            symboltype))))
-  (let ((iterfun (gensym "WPI")))
-    `(LET ((,iterfun (PACKAGE-ITERATOR-FUNCTION
+  (let ((iterfun (gensym "WPI-")))
+    `(LET ((,iterfun (SYS::PACKAGE-ITERATOR-FUNCTION
                        ,pack-list ',(remove-duplicates types))))
        (MACROLET ((,name () '(FUNCALL ,iterfun)))
          ,@body))))
@@ -218,7 +222,7 @@ Also the default packages to unlock by WITHOUT-PACKAGE-LOCK.")
 
 (defmacro doseq ((var seqform &optional resultform) &body body)
   (multiple-value-bind (body-rest declarations) (system::parse-body body)
-    (let ((seqvar (gensym)))
+    (let ((seqvar (gensym "SEQ-")))
       `(BLOCK NIL
          (LET ((,seqvar ,seqform))
            (LET ((,var NIL))
@@ -422,7 +426,7 @@ Also the default packages to unlock by WITHOUT-PACKAGE-LOCK.")
 
 (defmacro dohash ((keyvar valuevar HTform &optional resultform) &body body)
   (multiple-value-bind (body-rest declarations) (system::parse-body body)
-    (let ((HTvar (gensym)))
+    (let ((HTvar (gensym "HASH-TABLE-")))
       `(BLOCK NIL
          (LET ((,HTvar ,HTform))
            (LET ((,keyvar NIL) (,valuevar NIL))
