@@ -2,6 +2,11 @@
 
 (defun princ-error (c) (format t "~&[~A]: ~A~%" (type-of c) c))
 
+(defun post-compile-file-cleanup (file)
+  (delete-file file)
+  (delete-file (compile-file-pathname file))
+  #+clisp (delete-file (make-pathname :type "lib" :defaults file)))
+
 #+OLD-CLISP
 ;; Binding *ERROR-HANDLER* is a hammer technique for catching errors. It also
 ;; disables CLCS processing and thus breaks tests that rely on the condition
@@ -59,7 +64,11 @@
   #-(or CLISP AKCL GCL ECL ALLEGRO CMU) (lisp-implementation-type))
 
 (defvar *eval-method* :eval)
+(defvar *eval-out* nil)
+(defvar *eval-err* nil)
 (defun my-eval (form)
+  (when *eval-out* (get-output-stream-string *eval-out*))
+  (when *eval-err* (get-output-stream-string *eval-err*))
   (ecase *eval-method*
     (:eval (eval form))
     (:compile (funcall (compile nil `(lambda () ,form))))
@@ -69,23 +78,44 @@
                (error "eval: ~S; compile: ~S" e-value c-value))
              e-value))))
 
+;; compare 2 sequences:
+;; list of (pos first second);
+;; last 3 elements can be smaller length, tail1, tail2
+(defun diff-seq (s1 s2 &key (test #'eql))
+  (let ((pos 0))
+    (nconc
+     (delete nil
+             (map 'list (lambda (e1 e2)
+                          (prog1 (unless (funcall test e1 e2)
+                                   (list pos e1 e2))
+                            (incf pos)))
+                  s1 s2))
+     (let ((l1 (length s1)) (l2 (length s2)))
+       (cond ((< l1 l2) (list l1 nil (subseq s2 l1)))
+             ((< l2 l1) (list l2 (subseq s1 l2) nil)))))))
+
 (defgeneric pretty-compare (result my-result log)
   (:documentation "print a pretty comparison of two results")
   (:method ((result sequence) (my-result sequence) (log stream))
     (let ((pos (mismatch result my-result :test #'equalp)))
       (let ((*print-length* 10))
-        (flet ((pretty-tail-10 (seq)
-                 (if (and (> (length seq) (+ pos 10))
-                          (typep seq 'string))
-                     (concatenate 'string (subseq seq pos (+ pos 10)) "...")
-                     (subseq seq pos))))
-          (format log "~&Differ at position ~:D: ~S vs ~S~%CORRECT: ~S~%~7A: ~S~%"
-                  pos
-                  (if (< pos (length result)) (elt result pos) 'end-of-sequence)
-                  (if (< pos (length my-result)) (elt my-result pos) 'end-of-sequence)
-                  (pretty-tail-10 result)
-                  lisp-implementation
-                  (pretty-tail-10 my-result))))))
+        (if pos
+            (flet ((pretty-tail-10 (seq)
+                     (if (and (> (length seq) (+ pos 10))
+                              (typep seq 'string))
+                         (concatenate 'string (subseq seq pos (+ pos 10)) "...")
+                         (subseq seq pos))))
+              (format log "~&Differ at position ~:D: ~S vs ~S~%CORRECT: ~S~%~7A: ~S~%"
+                      pos
+                      (if (< pos (length result))
+                          (elt result pos) 'end-of-sequence)
+                      (if (< pos (length my-result))
+                          (elt my-result pos) 'end-of-sequence)
+                      (pretty-tail-10 result)
+                      lisp-implementation
+                      (pretty-tail-10 my-result)))
+            (format log "~&Type mismatch: ~S should be ~S~%"
+                    (type-of my-result) (type-of result))))))
   (:method ((result pathname) (my-result pathname) (log stream))
     (dolist (slot '(pathname-host pathname-device pathname-directory
                     pathname-name pathname-type pathname-version))
@@ -98,19 +128,34 @@
   "Print the object on its own line and return it. Used in many tests!"
   (fresh-line) (prin1 object) (terpri) object)
 
+(defun type-error-handler (err)
+  "Print the condition and THROW.
+Usage: (handler-bind ((type-error #'type-error-handler)) ...)"
+  (princ-error err)
+  (let ((da (type-error-datum err)) (et (type-error-expected-type err)))
+    (show (list :datum da :expected-type et) :pretty t)
+    (throw 'type-error-handler (typep da et))))
+
 (defvar *test-ignore-errors* t)
+(defvar *test-result-in-file* t
+  "T: CLISP-style: evaluation result in the file after the test form.
+NIL: sacla-style: forms should evaluate to non-NIL.")
 (defun do-test (stream log)
-  (let ((eof "EOF") (error-count 0) (total-count 0))
+  (let ((eof stream) (error-count 0) (total-count 0))
     (loop
-      (let ((form (read stream nil eof))
-            (result (read stream nil eof)))
+      (let ((form (read stream nil eof)) out err (result nil))
         (when (or (eq form eof) (eq result eof)) (return))
+        (if *test-result-in-file*
+            (setq result (read stream))
+            (setq form `(not ,form)))
         (incf total-count)
         (show form)
         (multiple-value-bind (my-result error-message)
             (if *test-ignore-errors*
                 (with-ignored-errors (my-eval form)) ; return ERROR on errors
                 (my-eval form)) ; don't disturb the condition system when testing it!
+          (setq out (and *eval-out* (get-output-stream-string *eval-out*))
+                err (and *eval-err* (get-output-stream-string *eval-err*)))
           (cond ((eql result my-result)
                  (format t "~&EQL-OK: ~S~%" result))
                 ((equal result my-result)
@@ -124,7 +169,9 @@
                              form result lisp-implementation
                              my-result error-message)
                  (pretty-compare result my-result log)
-                 (format log "~%"))))))
+                 (format log "~[~*~:;OUT:~%~S~%~]~[~*~:;ERR:~%~S~]~2%"
+                         (length out) out (length err) err)
+                 (force-output log))))))
     (values total-count error-count)))
 
 (defmacro check-ignore-errors (&body body)
@@ -163,7 +210,9 @@
         (when (or (eq form eof) (eq errtype eof)) (return))
         (incf total-count)
         (show form)
-        (let ((my-result (check-ignore-errors (my-eval form))))
+        (let ((my-result (check-ignore-errors (my-eval form)))
+              (out (and *eval-out* (get-output-stream-string *eval-out*)))
+              (err (and *eval-err* (get-output-stream-string *eval-err*))))
           (multiple-value-bind (typep-result typep-error)
               (ignore-errors (typep my-result errtype))
             (cond ((and (not typep-error) typep-result)
@@ -171,27 +220,40 @@
                   (t
                    (incf error-count)
                    (format t "~&ERROR!! ~S instead of ~S !~%" my-result errtype)
-                   (format log "~&Form: ~S~%CORRECT: ~S~%~7A: ~S~%"
-                               form errtype lisp-implementation
-                               my-result)))))))
+                   (format log "~&Form: ~S~%CORRECT: ~S~%~7A: ~S~%~
+                                ~[~*~:;OUT:~%~S~%~]~[~*~:;ERR:~%~S~]~2%"
+                               form errtype lisp-implementation my-result
+                               (length out) out (length err) err)
+                   (force-output log)))))))
     (values total-count error-count)))
 
 (defvar *run-test-tester* #'do-test)
+(defvar *run-test-type* "tst")
+(defvar *run-test-erg* "erg")
+(defvar *run-test-truename*)
 (defun run-test (testname
                  &key ((:tester *run-test-tester*) *run-test-tester*)
                       ((:ignore-errors *test-ignore-errors*)
                         *test-ignore-errors*)
                       ((:eval-method *eval-method*) *eval-method*)
                       (logname testname)
-                 &aux (logfile (merge-extension "erg" logname))
-                      error-count total-count)
-  (with-open-file (s (merge-extension "tst" testname) :direction :input)
+                 &aux (logfile (merge-extension *run-test-erg* logname))
+                      error-count total-count *run-test-truename*)
+  (with-open-file (s (merge-extension *run-test-type* testname)
+                     :direction :input)
+    (setq *run-test-truename* (truename s))
     (format t "~&~s: started ~s~%" 'run-test s)
     (with-open-file (log logfile :direction :output
-                                 #+(or CMU SBCL) :if-exists #+(or CMU SBCL) :supersede
+                                 #+(or CMU SBCL) :if-exists
+                                 #+(or CMU SBCL) :supersede
                                  #+ANSI-CL :if-exists #+ANSI-CL :new-version)
       (setq logfile (truename log))
-      (let ((*package* *package*) (*print-circle* t) (*print-pretty* nil))
+      (let* ((*package* *package*) (*print-circle* t) (*print-pretty* nil)
+             (*eval-err* (make-string-output-stream))
+             (*error-output* (make-broadcast-stream *error-output* *eval-err*))
+             (*eval-out* (make-string-output-stream))
+             (*standard-output* (make-broadcast-stream *standard-output*
+                                                       *eval-out*)))
         (setf (values total-count error-count)
               (funcall *run-test-tester* s log)))))
   (format t "~&~s: finished ~s (~:d error~:p out of ~:d test~:p)~%"
@@ -199,58 +261,46 @@
   (if (zerop error-count)
       (delete-file logfile)
       (format t "~s: see ~a~%" 'run-test logfile))
-  (values total-count error-count))
+  (list testname total-count error-count))
 
-(defmacro with-accumulating-errors ((error-count total-count) &body body)
-  (let ((err (gensym)) (tot (gensym)))
-    `(multiple-value-bind (,tot ,err) (progn ,@body)
-       (incf ,error-count ,err)
-       (incf ,total-count ,tot))))
-
-(defun run-files (files)
-  (let* ((error-count 0) (total-count 0)
-         (here (truename "./"))
-         (res (mapcar (lambda (file)
-                        (multiple-value-bind (tot err) (run-test file)
-                          (incf error-count err)
-                          (incf total-count tot)
-                          (list file err tot)))
-                      files)))
+(defun report-results (res)
+  "res = list of RUN-TEST return values (testname total-count error-count)"
+  (let ((error-count (reduce #'+ res :key #'third)))
     (format
-     t "~&~s: finished ~3d file~:p:~31T ~3:d error~:p out of~50T ~5:d test~:p~%"
-     'run-files (length files) error-count total-count)
-    (loop :for rec :in res :for count :upfrom 1 :do
-      (format t "~&[~d] ~25@a:~31T ~3:d error~:p out of~50T ~5:d test~:p~%"
+     t "~&finished ~3d file~:p:~31T ~3:d error~:p out of~50T ~5:d test~:p~%"
+     (length res) error-count (reduce #'+ res :key #'second))
+    (loop :with here = (truename "./") :for rec :in res :for count :upfrom 1 :do
+      (format t "~&~3d ~25@a:~31T ~3:d error~:p out of~50T ~5:d test~:p~%"
               count (enough-namestring (first rec) here)
-              (second rec) (third rec)))
-    (values error-count total-count res)))
+              (third rec) (second rec)))
+    error-count))
 
 (defun run-some-tests (&key (dirlist '("./"))
                        ((:eval-method *eval-method*) *eval-method*))
   (let ((files (mapcan (lambda (dir)
-                         (directory (make-pathname :name :wild :type "tst"
+                         (directory (make-pathname :name :wild
+                                                   :type *run-test-type*
                                                    :defaults dir)))
                        dirlist)))
-    (if files (run-files files)
-        (warn "no TST files in directories ~S" dirlist))))
+    (if files (report-results (mapcar #'run-test files))
+        (warn "no ~S files in directories ~S" *run-test-type* dirlist))))
 
 (defun run-all-tests (&key (disable-risky t)
                       ((:eval-method *eval-method*) *eval-method*))
-  (let ((error-count 0) (total-count 0)
+  (let ((res ())
         #+CLISP (custom:*load-paths* nil)
-        #+CLISP (custom:*warn-on-floating-point-contagion* nil)
-        #+CLISP (custom:*warn-on-floating-point-rational-contagion* nil)
         (*features* (if disable-risky *features*
                         (cons :enable-risky-tests *features*))))
-    (dolist (ff `(#-(or AKCL ECL)   "alltest"
+    (dolist (ff '(#-(or AKCL ECL)   "alltest"
                                     "array"
                   #-OpenMCL         "backquot"
                   #+CLISP           "bin-io"
                   #-(and AKCL (not GCL)) "characters"
                   #+(or CLISP ALLEGRO CMU OpenMCL LISPWORKS) "clos"
-                  #+CLISP ,@(unless disable-risky '("defhash"))
+                  #+CLISP           "defhash"
                   #+(and CLISP UNICODE) "encoding"
                                     "eval20"
+                  #+CLISP           "ext-clisp"
                   #+(and CLISP FFI) "ffi"
                                     "floeps"
                   #-OpenMCL         "format"
@@ -266,6 +316,7 @@
                                     "lists154"
                                     "lists155"
                                     "lists156"
+                                    "list-set"
                   #+(or CLISP GCL ALLEGRO CMU SBCL OpenMCL LISPWORKS) "loop"
                                     "macro8"
                                     "map"
@@ -276,6 +327,7 @@
                   #+(or XCL CLISP)  "path"
                   #+XCL             "readtable"
                   #-CMU             "setf"
+                  #+(and CLISP SOCKETS) "socket"
                                     "steele7"
                   #-ALLEGRO         "streams"
                                     "streamslong"
@@ -286,27 +338,24 @@
                   #+XCL             "tprint"
                   #+XCL             "tread"
                                     "type"
+                  #-(or)            "unportable"
                   #+CLISP           "weak"
                   #+(or CLISP ALLEGRO CMU19 OpenMCL LISPWORKS) "weakhash"
                   #+(or CLISP LISPWORKS) "weakhash2"))
-      (with-accumulating-errors (error-count total-count) (run-test ff)))
-    ;; fails on amd64 - disable for now...
-    ;;(with-accumulating-errors (error-count total-count)
-    ;;  (run-test "bind" :eval-method :eval :logname "bind-eval"))
-    (with-accumulating-errors (error-count total-count)
-      (run-test "bind" :eval-method :compile :logname "bind-compile"))
+      (push (run-test ff) res))
+    (unless disable-risky ; fails on amd64 - disable for now...
+      (push (run-test "bind" :eval-method :eval :logname "bind-eval") res))
+    (push (run-test "bind" :eval-method :compile :logname "bind-compile") res)
     #+(or CLISP ALLEGRO CMU SBCL LISPWORKS)
-    (dotimes (i 20)
-      (with-accumulating-errors (error-count total-count)
-        (run-test "weakptr")))
+    (let ((tmp (list "weakptr" 0 0)))
+      (push tmp res)
+      (dotimes (i 20)
+        (let ((weak-res (run-test "weakptr")))
+          (incf (second tmp) (second weak-res))
+          (incf (third tmp) (third weak-res)))))
     #+(or CLISP ALLEGRO CMU LISPWORKS)
-    (with-accumulating-errors (error-count total-count)
-      (run-test "conditions" :ignore-errors nil))
+    (push (run-test "conditions" :ignore-errors nil) res)
     #+CLISP
-    (with-accumulating-errors (error-count total-count)
-      (run-test "restarts" :ignore-errors nil))
-    (with-accumulating-errors (error-count total-count)
-      (run-test "excepsit" :tester #'do-errcheck))
-    (format t "~&~s: grand total: ~:d error~:p out of ~:d test~:p~%"
-            'run-all-tests error-count total-count)
-    (values total-count error-count)))
+    (push (run-test "restarts" :ignore-errors nil) res)
+    (push (run-test "excepsit" :tester #'do-errcheck) res)
+    (report-results (nreverse res))))
