@@ -4,9 +4,11 @@
 #
 # To verify:   find . -type f -perm +111 -print
 
-find . -type f -perm +111 '(' \
+find . -type f -follow -perm +111 '(' \
      -name '*.in' \
   -o -name '*.xml' \
+  -o -name '*.html' \
+  -o -name '*.png' \
   -o -name '*.xsl' \
   -o -name '*.m4' \
   -o -name '*.h' \
@@ -14,10 +16,11 @@ find . -type f -perm +111 '(' \
   -o -name '*.lisp' \
   -o -name '*.tst' \
   -o -name '*.bat' \
+  -o -name 'link.sh' \
   -o -name '.cvsignore' \
   -o -name 'Makefile' \
   -o -name 'README' \
   ')' \
-  -exec chmod a-x '{}' ';'
+  -exec chmod -c a-x '{}' ';'
 
-find . -name configure -exec chmod +x '{}' ';'
+find . -name configure -exec chmod -c +x '{}' ';'
